@@ -28,9 +28,9 @@ class mcp_warn
 	var $p_master;
 	var $u_action;
 
-	function mcp_warn(&$p_master)
+	function __construct($p_master)
 	{
-		$this->p_master = &$p_master;
+		$this->p_master = $p_master;
 	}
 
 	function main($id, $mode)
